"use strict";
exports.__esModule = true;
var schematics_1 = require("@angular-devkit/schematics");
var core_1 = require("@angular-devkit/core");
var tasks_1 = require("@angular-devkit/schematics/tasks");
var devDependencies = [
    {
        name: 'wallaby-webpack',
        version: '*'
    },
    {
        name: 'angular2-template-loader',
        version: '^0.6.2'
    }
];
function default_1(_options) {
    return schematics_1.chain([
        addConfigFile(),
        addBootstrapFile(),
        modifyTsconfigExcludeFile(),
        addLoadersToPackageJson(),
        addPackageInstallTask()
    ]);
}
exports["default"] = default_1;
function addConfigFile() {
    return function (tree, context) {
        var nx = containNxScematics(tree);
        var importCompilerOptions, importAlias;
        var wallabyConfigSourceFileName = nx ? '/nx/wallaby.js' : '/wallaby.js';
        var wallabyConfigFileName = '/wallaby.js';
        if (nx) {
            importCompilerOptions = createCompilerOptions(getProjectLibs(tree));
            importAlias = createAlias(tree);
        }
        else {
            importCompilerOptions = '';
            importAlias = '';
        }
        var files = schematics_1.url("../../files")(context);
        var scriptContent = files
            .read(wallabyConfigSourceFileName)
            .toString('utf-8');
        var createContent = core_1.template(scriptContent)({
            importCompilerOptions: importCompilerOptions,
            importAlias: importAlias
        });
        if (!tree.exists(wallabyConfigFileName)) {
            tree.create(wallabyConfigFileName, createContent);
        }
        return tree;
    };
}
function addBootstrapFile() {
    return function (tree, context) {
        var nx = containNxScematics(tree);
        var wallabyBootstrapSourceFileName = nx
            ? '/nx/wallabyTest.ts'
            : '/src/wallabyTest.ts';
        var wallabyBootstrapFileName = nx
            ? '/wallabyTest.ts'
            : '/src/wallabyTest.ts';
        var files = schematics_1.url("../../files")(context);
        var scriptContent = files
            .read(wallabyBootstrapSourceFileName)
            .toString('utf-8');
        if (!tree.exists(wallabyBootstrapFileName)) {
            tree.create(wallabyBootstrapFileName, scriptContent);
        }
        return tree;
    };
}
function modifyTsconfigExcludeFile() {
    return function (tree) {
        var nx = containNxScematics(tree);
        if (tree.exists('/src/tsconfig.app.json')) {
            var tsConfigSource = tree
                .read('/src/tsconfig.app.json')
                .toString('utf-8');
            var json = JSON.parse(tsConfigSource);
            if (json['exclude']) {
                if (!json['exclude'].includes('wallabyTest.ts')) {
                    json['exclude'].push('wallabyTest.ts');
                }
            }
            tree.overwrite('/src/tsconfig.app.json', JSON.stringify(json, null, 2));
        }
        else if (nx) {
        }
        else {
            throw new schematics_1.SchematicsException("Could not find tsconfig.app.json.");
        }
        return tree;
    };
}
function addLoadersToPackageJson() {
    return function (tree) {
        devDependencies.forEach(function (dependency) {
            addDependencyToPackageJson(tree, 'devDependencies', dependency.name, dependency.version);
        });
        return tree;
    };
}
function addPackageInstallTask() {
    return function (tree, context) {
        var depNames = devDependencies.map(function (d) { return d.name; }).join(' ');
        context.addTask(new tasks_1.NodePackageInstallTask({
            packageName: depNames
        }));
        return tree;
    };
}
/**
 * Adds a package to the package.json
 */
function addDependencyToPackageJson(tree, type, pkg, version) {
    if (version === void 0) { version = 'latest'; }
    if (tree.exists('/package.json')) {
        var sourceText = tree.read('/package.json').toString('utf-8');
        var json = JSON.parse(sourceText);
        if (!json[type]) {
            json[type] = {};
        }
        if (!json[type][pkg]) {
            json[type][pkg] = version;
        }
        tree.overwrite('/package.json', JSON.stringify(json, null, 2));
    }
    return tree;
}
exports.addDependencyToPackageJson = addDependencyToPackageJson;
function containNxScematics(tree) {
    var sourceText = tree.read('/package.json').toString('utf-8');
    var packageJson = JSON.parse(sourceText);
    return !!packageJson.dependencies['@nrwl/nx'];
}
function getProjectLibs(tree) {
    var sourceText = tree.read('/angular.json').toString('utf-8');
    var packageJson = JSON.parse(sourceText);
    var projects = { apps: [], libs: [] };
    for (var project in packageJson.projects) {
        if (project.includes('-e2e')) {
            continue;
        }
        var projectInfo = packageJson.projects[project];
        if (projectInfo.projectType === 'application') {
            projects.apps.push(project);
        }
        if (projectInfo.projectType === 'library') {
            projects.libs.push(project);
        }
    }
    return projects;
}
function createCompilerOptions(defaultProject) {
    var compilerOptions = [];
    compilerOptions = compilerOptions.concat(defaultProject.apps.map(function (app) {
        return "require('./apps/" + app + "/tsconfig.spec.json').compilerOptions";
    }));
    compilerOptions = compilerOptions.concat(defaultProject.libs.map(function (lib) {
        return "require('./libs/" + lib + "/tsconfig.spec.json').compilerOptions";
    }));
    return compilerOptions.join(',');
}
function createAlias(tree) {
    var sourceText = tree.read('/tsconfig.json').toString('utf-8');
    var tsconfig = JSON.parse(sourceText);
    var alias = [];
    for (var path in tsconfig.compilerOptions.paths) {
        if (!path.includes('*')) {
            var value = tsconfig.compilerOptions.paths[path];
            alias.push("'" + path + "': path.join(wallaby.projectCacheDir, '" + value + "' )");
        }
    }
    if (alias.length === 0) {
        return '';
    }
    return "alias: {\n    " + alias.join(',') + "\n  }";
}
//# sourceMappingURL=index.js.map